// Stat Bar Overlays Library Demo
// Version 0.6 - 2013-10-05
// Create by DarkCampainger

// This library allows you to quickly create stat bar overlays
//   for health, mana, experience, and more. Tell it which
//   variables to track when it's created, and then all you have to
//   do is call Update()

// See included Reference.dm for process reference

#define FILE_DIR demo_icons

mob
	icon = 'DemoPlayer.dmi'

	var
		// We define our stats and their max values
		health = 100
		health_max = 100

		mana = 5
		mana_max = 10

		// We define the variables to hold the references to the bars
		// We also mark them as 'tmp' to ensure they aren't saved by the default Write() proc
		tmp
			bar_overlay
				health_bar
				mana_bar

	Login()

		// Create a health bar with 14 states based on the "health" and "health_max" variables
		// The icon 'HealthBar.dmi' should have states "1" through "14", with 1 being for 0%
		// We place it at a pixel_y offset so it doesn't cover the player
		health_bar = new(src, 'DemoHealthBar.dmi', 14, "health", "health_max", pixel_y = -8)

		// Create a mana bar, using the properties of the /bar_overlay/mana_bar type (defined below)
		mana_bar = new/bar_overlay/mana_bar(src)

		// Do BYOND's default mob/Login() action (place the player on the first non-dense tile it finds)
		..()

	verb
		Set_Health(n as num)
			health = n

			// To update a stat bar, simply call its Update() process
			// While you could run this off a timed loop, it's more efficient
			//   to call it only when you change it's associated stat
			health_bar.Update()

		Set_Health_Max(n as num)
			health_max = n
			health_bar.Update()

		Set_Mana(n as num)
			mana = n
			mana_bar.Update()

		Set_Mana_Max(n as num)
			mana_max = n
			mana_bar.Update()

		Randomize()
			health_max = rand(1,1000)
			health = rand(0,health_max)
			health_bar.Update()

			mana_max = rand(1,1000)
			mana = rand(0,mana_max)
			mana_bar.Update()

		Cast_Bolt()
			var/cost = max(round(mana_max/10), 1)
			if(mana >= cost)
				mana -= cost
				mana_bar.Update()
				missile('DemoBolt.dmi', src, pick(oview(6,src)))
			else
				src<<"You need at least [cost] mana to cast that!"

		Show_Mana()
			mana_bar.Show(12)

	Stat()
		stat("Health: [src.health]/[src.health_max] ([round(100*src.health/src.health_max)]%)")
		stat("Mana: [src.mana]/[src.mana_max] ([round(100*src.mana/src.mana_max)]%)")


bar_overlay
	// You can also define a stat bar's properties directly in its own child type
	mana_bar
		// Uses the 'ManaBar.dmi' icon
		icon = 'DemoManaBar.dmi'
		// The icon_state names are prefixed with "Mana"
		state_prefix = "Mana"
		// It tracks the variable "mana" out of "mana_max"
		var_stat = "mana"
		var_max = "mana_max"
		// Offset the pixel_y of the overlay by -16 pixels
		pixel_y = -16
		// We set the bar to "timeout" and auto-hide if it hasn't been updated in 12 ticks
		timeout = 12

		// We don't define the number of states (10), but instead let the system figure it out
		// This is technically less efficient, but allows us to fiddle with the icons without
		//   having to update the definition for the stat bar all the time.


turf/icon = 'DemoGrass.dmi'
world
	maxx=17
	maxy=17
	maxz=1